@@ -3,7 +3,6 @@ package resources
 import (
 	"fmt"
 	"testing"
-	"time"
 
 	"github.com/google/go-cmp/cmp"
 	routev1 "github.com/openshift/api/route/v1"
@@ -62,7 +61,7 @@ func TestMakeRoute(t *testing.T) {
 						OpenShiftIngressNamespaceLabelKey: "default",
 					},
 					Annotations: map[string]string{
-						TimeoutAnnotation: defaultTimeout,
+						TimeoutAnnotation: DefaultTimeout,
 					},
 					Namespace: lbNamespace,
 					Name:      routeName0,
@@ -75,7 +74,7 @@ func TestMakeRoute(t *testing.T) {
 						Weight: ptr.Int32(100),
 					},
 					Port: &routev1.RoutePort{
-						TargetPort: intstr.FromString(KourierHTTPPort),
+						TargetPort: intstr.FromString(HTTPPort),
 					},
 					TLS: &routev1.TLSConfig{
 						Termination:                   routev1.TLSTerminationEdge,
@@ -97,44 +96,6 @@ func TestMakeRoute(t *testing.T) {
 			ingress: ingress(withRules(rule(withHosts([]string{localDomain, externalDomain}), withLocalVisibilityRule))),
 			want:    []*routev1.Route{},
 		},
-		{
-			name: "valid, with timeout",
-			ingress: ingress(withRules(
-				rule(withHosts([]string{localDomain, externalDomain}), withTimeout(1*time.Hour))),
-			),
-			want: []*routev1.Route{{
-				ObjectMeta: metav1.ObjectMeta{
-					Labels: map[string]string{
-						networking.IngressLabelKey:        "ingress",
-						serving.RouteLabelKey:             "route1",
-						serving.RouteNamespaceLabelKey:    "default",
-						OpenShiftIngressLabelKey:          "ingress",
-						OpenShiftIngressNamespaceLabelKey: "default",
-					},
-					Annotations: map[string]string{
-						TimeoutAnnotation: "3600s",
-					},
-					Namespace: lbNamespace,
-					Name:      routeName0,
-				},
-				Spec: routev1.RouteSpec{
-					Host: externalDomain,
-					To: routev1.RouteTargetReference{
-						Kind:   "Service",
-						Name:   lbService,
-						Weight: ptr.Int32(100),
-					},
-					Port: &routev1.RoutePort{
-						TargetPort: intstr.FromString(KourierHTTPPort),
-					},
-					TLS: &routev1.TLSConfig{
-						Termination:                   routev1.TLSTerminationEdge,
-						InsecureEdgeTerminationPolicy: routev1.InsecureEdgeTerminationPolicyAllow,
-					},
-					WildcardPolicy: routev1.WildcardPolicyNone,
-				},
-			}},
-		},
 		{
 			name: "valid, multiple rules",
 			ingress: ingress(withRules(
@@ -151,7 +112,7 @@ func TestMakeRoute(t *testing.T) {
 						OpenShiftIngressNamespaceLabelKey: "default",
 					},
 					Annotations: map[string]string{
-						TimeoutAnnotation: defaultTimeout,
+						TimeoutAnnotation: DefaultTimeout,
 					},
 					Namespace: lbNamespace,
 					Name:      routeName0,
@@ -164,7 +125,7 @@ func TestMakeRoute(t *testing.T) {
 						Weight: ptr.Int32(100),
 					},
 					Port: &routev1.RoutePort{
-						TargetPort: intstr.FromString(KourierHTTPPort),
+						TargetPort: intstr.FromString(HTTPPort),
 					},
 					TLS: &routev1.TLSConfig{
 						Termination:                   routev1.TLSTerminationEdge,
@@ -182,7 +143,7 @@ func TestMakeRoute(t *testing.T) {
 						OpenShiftIngressNamespaceLabelKey: "default",
 					},
 					Annotations: map[string]string{
-						TimeoutAnnotation: defaultTimeout,
+						TimeoutAnnotation: DefaultTimeout,
 					},
 					Namespace: lbNamespace,
 					Name:      routeName1,
@@ -195,7 +156,7 @@ func TestMakeRoute(t *testing.T) {
 						Weight: ptr.Int32(100),
 					},
 					Port: &routev1.RoutePort{
-						TargetPort: intstr.FromString(KourierHTTPPort),
+						TargetPort: intstr.FromString(HTTPPort),
 					},
 
 					TLS: &routev1.TLSConfig{
@@ -222,7 +183,7 @@ func TestMakeRoute(t *testing.T) {
 						OpenShiftIngressNamespaceLabelKey: "default",
 					},
 					Annotations: map[string]string{
-						TimeoutAnnotation: defaultTimeout,
+						TimeoutAnnotation: DefaultTimeout,
 					},
 					Namespace: lbNamespace,
 					Name:      routeName1,
@@ -235,7 +196,7 @@ func TestMakeRoute(t *testing.T) {
 						Weight: ptr.Int32(100),
 					},
 					Port: &routev1.RoutePort{
-						TargetPort: intstr.FromString(KourierHTTPPort),
+						TargetPort: intstr.FromString(HTTPPort),
 					},
 
 					TLS: &routev1.TLSConfig{
@@ -253,6 +214,84 @@ func TestMakeRoute(t *testing.T) {
 			),
 			wantErr: ErrNoValidLoadbalancerDomain,
 		},
+		{
+			name: "valid, passthrough by annotation",
+			ingress: ingress(withPassthroughAnnotation, withRules(
+				rule(withHosts([]string{localDomain, externalDomain}))),
+			),
+			want: []*routev1.Route{{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						networking.IngressLabelKey:        "ingress",
+						serving.RouteLabelKey:             "route1",
+						serving.RouteNamespaceLabelKey:    "default",
+						OpenShiftIngressLabelKey:          "ingress",
+						OpenShiftIngressNamespaceLabelKey: "default",
+					},
+					Annotations: map[string]string{
+						TimeoutAnnotation:                DefaultTimeout,
+						EnablePassthroughRouteAnnotation: "true",
+					},
+					Namespace: lbNamespace,
+					Name:      routeName0,
+				},
+				Spec: routev1.RouteSpec{
+					Host: externalDomain,
+					To: routev1.RouteTargetReference{
+						Kind:   "Service",
+						Name:   lbService,
+						Weight: ptr.Int32(100),
+					},
+					Port: &routev1.RoutePort{
+						TargetPort: intstr.FromString(HTTPSPort),
+					},
+					TLS: &routev1.TLSConfig{
+						Termination:                   routev1.TLSTerminationPassthrough,
+						InsecureEdgeTerminationPolicy: routev1.InsecureEdgeTerminationPolicyRedirect,
+					},
+					WildcardPolicy: routev1.WildcardPolicyNone,
+				},
+			}},
+		},
+		{
+			name: "valid, passthrough by BYO cert",
+			ingress: ingress(
+				withTLS(networkingv1alpha1.IngressTLS{Hosts: []string{"custom.example.com"}, SecretName: "someSecretName"}),
+				withRules(rule(withHosts([]string{localDomain, externalDomain}))),
+			),
+			want: []*routev1.Route{{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						networking.IngressLabelKey:        "ingress",
+						serving.RouteLabelKey:             "route1",
+						serving.RouteNamespaceLabelKey:    "default",
+						OpenShiftIngressLabelKey:          "ingress",
+						OpenShiftIngressNamespaceLabelKey: "default",
+					},
+					Annotations: map[string]string{
+						TimeoutAnnotation: DefaultTimeout,
+					},
+					Namespace: lbNamespace,
+					Name:      routeName0,
+				},
+				Spec: routev1.RouteSpec{
+					Host: externalDomain,
+					To: routev1.RouteTargetReference{
+						Kind:   "Service",
+						Name:   lbService,
+						Weight: ptr.Int32(100),
+					},
+					Port: &routev1.RoutePort{
+						TargetPort: intstr.FromString(HTTPSPort),
+					},
+					TLS: &routev1.TLSConfig{
+						Termination:                   routev1.TLSTerminationPassthrough,
+						InsecureEdgeTerminationPolicy: routev1.InsecureEdgeTerminationPolicyRedirect,
+					},
+					WildcardPolicy: routev1.WildcardPolicyNone,
+				},
+			}},
+		},
 	}
 
 	for _, test := range tests {
@@ -311,6 +350,12 @@ func rule(options ...ruleOption) networkingv1alpha1.IngressRule {
 
 type ingressOption func(*networkingv1alpha1.Ingress)
 
+func withTLS(tls ...networkingv1alpha1.IngressTLS) ingressOption {
+	return func(ing *networkingv1alpha1.Ingress) {
+		ing.Spec.TLS = tls
+	}
+}
+
 func withRules(rules ...networkingv1alpha1.IngressRule) ingressOption {
 	return func(ing *networkingv1alpha1.Ingress) {
 		ing.Spec.Rules = rules
@@ -326,6 +371,15 @@ func withDisabledAnnotation(ing *networkingv1alpha1.Ingress) {
 	ing.SetAnnotations(annos)
 }
 
+func withPassthroughAnnotation(ing *networkingv1alpha1.Ingress) {
+	annos := ing.GetAnnotations()
+	if annos == nil {
+		annos = map[string]string{}
+	}
+	annos[EnablePassthroughRouteAnnotation] = "true"
+	ing.SetAnnotations(annos)
+}
+
 func withLBInternalDomain(domain string) ingressOption {
 	return func(ing *networkingv1alpha1.Ingress) {
 		ing.Status.PublicLoadBalancer.Ingress[0].DomainInternal = domain
@@ -343,13 +397,3 @@ func withHosts(hosts []string) ruleOption {
 		rule.Hosts = hosts
 	}
 }
-
-func withTimeout(timeout time.Duration) ruleOption {
-	return func(rule *networkingv1alpha1.IngressRule) {
-		rule.HTTP = &networkingv1alpha1.HTTPIngressRuleValue{
-			Paths: []networkingv1alpha1.HTTPIngressPath{{
-				DeprecatedTimeout: &metav1.Duration{Duration: timeout},
-			}},
-		}
-	}
-}
